#include "mpi.h"
#include "Python.h"
#include "numpy/arrayobject.h"

/* ----------------------------------------------- */
/* This assumes g77 style mangling for FORTRAN     */
/* ----------------------------------------------- */
extern void something_(int*,double*,int*);

static PyObject* call_fortran(PyObject* self,PyObject* args) {
  long lcomm = 0;
  PyObject* pyA = 0;
  double* A = 0;
  int n = 0;
  MPI_Fint f_comm;
  MPI_Fint f_n;

  /* ----------------------------------------------- */
  /* First grab the communicator as a Python int (C  */
  /* long) and whatever other arguments you want     */
  /* ----------------------------------------------- */
  PyArg_ParseTuple(args,"lO",&lcomm,&pyA);
  if ( PyErr_Occurred() ) return 0;

  /* ----------------------------------------------- */
  /* Grab a contiguous array                         */
  /* ----------------------------------------------- */
  PyArray_As1D(&pyA, (char**)(&A), &n, 'd');
  if ( PyErr_Occurred() ) return 0;

  /* ----------------------------------------------- */
  /* Call FORTRAN                                    */
  /* ----------------------------------------------- */
  f_comm = MPI_Comm_c2f((MPI_Comm)lcomm);
  f_n = n;
  something_(&f_comm,A,&f_n);

  /* ----------------------------------------------- */
  /* Clean up                                        */
  /* ----------------------------------------------- */
  Py_XDECREF(pyA);
  Py_INCREF(Py_None);
  return Py_None;
}

static PyMethodDef methods[] = {
  {"call_fortran",call_fortran,METH_VARARGS,"call_fortran(comm,array)\n\nA simple FORTRAN routine takes a communicator and an array"},
  {0,0}
};

void initexample(void) {
  Py_InitModule("example",methods);
  import_array(); /* Magic handshake for Numeric */
}
