#include <stdio.h>
#include "mpi.h"
#include "pstencil.h"

/**************************************************************************/
/* GLOBAL **************           first           ************************/
/**************************************************************************/
/* Find global index of first element                                     */
/**************************************************************************/
int first(double* field, int n) {
  int sum;
  /* We add up the sizes using SCAN and figure out our first element position */
  MPI_Scan(&n, &sum, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD);

  return sum-n;
}

/**************************************************************************/
/* GLOBAL **************         threepoint        ************************/
/**************************************************************************/
/* Super simple 3-point stencil                                           */
/**************************************************************************/
void threepoint(double left, double* field, int n, double right) {
  int i;
  double t;
  int rank;
  int size;

  MPI_Status status;

  MPI_Comm_rank(MPI_COMM_WORLD,&rank); 
  MPI_Comm_size(MPI_COMM_WORLD,&size);

  /* Get local left and right boundaries */
  if ( size != 1 ) {
    if ( rank == 0 ) {
      MPI_Sendrecv(&field[n-1],1,MPI_DOUBLE,rank+1,77,
		   &right,1,MPI_DOUBLE,rank+1,77,
		   MPI_COMM_WORLD,&status);
      printf("rank 0 sent %g got %g\n",field[n-1],right);
    } else if ( rank == (size-1) ) {
      MPI_Sendrecv(&field[0],1,MPI_DOUBLE,rank-1,77,
		   &left,1,MPI_DOUBLE,rank-1,77,
		   MPI_COMM_WORLD,&status);
      printf("rank %d sent %g got %g\n",rank,field[0],left);
    } else {
      MPI_Sendrecv(&field[n-1],1,MPI_DOUBLE,rank+1,77,
		   &left,1,MPI_DOUBLE,rank-1,77,
		   MPI_COMM_WORLD,&status);
      printf("rank %d sent %g got %g\n",rank,field[n-1],left);
      MPI_Sendrecv(&field[0],1,MPI_DOUBLE,rank-1,77,
		   &right,1,MPI_DOUBLE,rank+1,77,
		   MPI_COMM_WORLD,&status);
      printf("rank %d sent %g got %g\n",rank,field[0],right);
    }
  }
  MPI_Barrier(MPI_COMM_WORLD);
  printf("rank: %d of %d left: %g right: %g\n",rank,size,left,right);


  for(i=0;i<n-1;++i) {
    t = (left+field[i]+field[i+1])/3.0;
    left = field[i];
    field[i] = t;
  }
  field[n-1] = (left+field[n-1]+right)/3.0;
  return;
}


/**************************************************************************/
/* GLOBAL **************        set_sources        ************************/
/**************************************************************************/
/* Set source values in the field                                         */
/**************************************************************************/
void set_sources(int m, int* indices, double* values, double* field, int n, int first) {
  int i;
  for(i=0;i<m;++i) {
    printf("indices[%d] = %d\n",i,indices[i]);
    if ( indices[i] >= first && indices[i] < first+n ) {
      printf("Set global %d to %g (local %d)\n",indices[i],values[i],indices[i]-first);
      field[indices[i]-first] = values[i];
    }
  }
      
  return;
}
