#include "Python.h"
#include "numpy/arrayobject.h"

#include "pstencil.h"

static PyObject* first_wrapper(PyObject* self, PyObject* args) {
  PyObject* field = 0;
  double* field_pointer = 0;
  int field_size = 0;

  /* Get arguments */
  PyArg_ParseTuple(args,"O",&field);
  if ( PyErr_Occurred() ) return 0;

  /* Convert to 1D arrays and pass along */
  PyArray_As1D(&field,(char**)(&field_pointer),&field_size,'d');
  if ( PyErr_Occurred() ) return 0;

  return PyInt_FromLong(first(field_pointer,field_size));
}

static PyObject* set_sources_wrapper(PyObject* self, PyObject* args) {
  PyObject* indices = 0;
  PyObject* values = 0;
  PyObject* field = 0;

  int* indices_pointer = 0;
  double* values_pointer = 0;
  double* field_pointer = 0;
  int global_low = 0;

  int indices_size = 0;
  int values_size = 0;
  int field_size = 0;

  /* Get arguments */
  PyArg_ParseTuple(args,"OOOi",&indices,&values,&field,&global_low);
  if ( PyErr_Occurred() ) return 0;

  /* Convert to 1D arrays and pass along */
  PyArray_As1D(&indices,(char**)(&indices_pointer),&indices_size,'d');
  if ( PyErr_Occurred() ) return 0;
  PyArray_As1D(&values,(char**)(&values_pointer),&values_size,'d');
  if ( PyErr_Occurred() ) return 0;
  PyArray_As1D(&field,(char**)(&field_pointer),&field_size,'d');
  if ( PyErr_Occurred() ) return 0;
  
  if ( indices_size != values_size ) {
    PyErr_SetString(PyExc_ValueError,"value and index sizes don't match");
    return 0;
  }

  set_sources(indices_size,indices_pointer,values_pointer,field_pointer,field_size,global_low);

  Py_DECREF(indices);
  Py_DECREF(values);
  Py_DECREF(field);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyObject* threepoint_wrapper(PyObject* self, PyObject* args) {
  double left;
  PyObject* field = 0;
  double right;

  PyObject* field_object = 0;
  double* field_pointer = 0;
  int field_size;
  
  /* Get arguments */
  PyArg_ParseTuple(args,"dOd",&left,&field,&right);
  if ( PyErr_Occurred() ) return 0;

  /* Convert numeric field to double* and size */
  field_object = field;
  PyArray_As1D(&field_object,(char**)(&field_pointer),&field_size,'d');
  
  threepoint(left,field_pointer,field_size,right);

  Py_INCREF(Py_None);
  return Py_None;
}

static PyMethodDef methods[] = {
  {"first",first_wrapper,METH_VARARGS,"first(field)"},
  {"threepoint",threepoint_wrapper,METH_VARARGS,"threepoint(left_bc,field,right_bc)"},
  {"set_sources",set_sources_wrapper,METH_VARARGS,"set_sources(indices,values,field)"},
  {0,0}
};

void initpstencil(void) {
  Py_InitModule("pstencil",methods);
  import_array(); /* Magic handshake for Numeric */
}
